//   CPmap.h -*- C++ -*- Completely Positive Maps in Hilbert space. 
//     
//   Copyright (C) 2007  Todd Brun, Bilal Shaw
//   
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//   
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//   
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//   ----------------------------------------------------------------------
//   If you improve the code or make additions to it, or if you have
//   comments or suggestions, please contact us:
//
//  
//
//   Dr. Todd Brun
//   Communication Sciences Institute           Tel    +001-213-740-3503
//   Department of Electrical Engineering       FAX    +001-213-740-8729 
//   University of Southern California          email  tbrun@usc.edu
//   3740 McClintock Ave, EEB 502
//   Los Angeles, CA, 90089-2565
//   USA			        
//
//   Bilal Shaw                                Tel    +001-213-740-5564
//   Communication Sciences Institute          email  bilalsha@usc.edu
//   Department of Electrical Engineering
//   University of Southern California
//   3740 McClintock Ave, EEB 514
//   Los Angeles, CA, 90089
/////////////////////////////////////////////////////////////////////////////

#ifndef CPmap_H 
#define CPmap_H

#include "Operator.h"
#include "Complex.h"


class CPmap{
  
  public:
  
  CPmap(); // Default constructor
  
  CPmap(int, Operator*);
  //The first argument gives the number of Kraus operators and the 
  // second argument is a pointer to an array of Kraus operators.
  
  CPmap(const CPmap& A);
  // copy constructor
  
  ~CPmap(); // Destructor
  
  CPmap& operator=(const CPmap&); //assignment operator
  
  CPmap& operator*=(CPmap&);//multiplication by a CPmap 
  CPmap& operator*=(const Complex&);//multiplication by a complex z*A[k]
  CPmap& operator*=(double);//multiplication by a real x*A[k] 							
  CPmap& operator*=(ImaginaryUnit);// i*A[k]
  CPmap& operator+=(CPmap&);//add a CPmap
  CPmap& operator-=(CPmap&);//subtract a CPmap									

  friend CPmap operator*(const CPmap&, CPmap&); //A[k]*B[j]
  friend CPmap operator*(const Complex&, const CPmap&); // z*A[k] 
  friend CPmap operator*(const CPmap&, const Complex&); // A[k]*z
  friend CPmap operator*(double, CPmap&); //x*A[k]
  friend CPmap operator*(CPmap&, double); // A[k]*x
  friend CPmap operator+(const CPmap&, CPmap&);//A[k]+B[k]
  friend CPmap operator-(const CPmap&, CPmap&);//A[k]-B[k]
  friend CPmap operator+(const CPmap&); // unary +A[k]
  friend CPmap operator-(const CPmap&); // unary -A[k]

  // accessor and mutator functions for setting the size of a CPmap
  int getcpmapSize(){return cpmapSize;};
  void setcpmapSize(int n){cpmapSize = n;};
  Operator* getcpmapPointer(){return cpmapPointer;};
  
  // multiplying a matrix by a CPmap should be written in Matrix.
  // We should make CPmap a friend class of Matrix, so that it can
  // access the routines in Matrix
  
  protected:
  
  void copy(const CPmap&);
  void free();
  int cpmapSize; // Size of our CPmap.  The number of Kraus operators
  Operator* cpmapPointer;// Pointer to array of Kraus operators.
  
  friend class Matrix; // 
};

#endif