// qwalk3.cc

// Quantum random walk.  One PARTICLE (position) degree of freedom, n SPINs
// (the quantum "coins").  Implements model of Aharonov, Ambainis, Kempe, etc.,
// with variations in the number of coins, intended to probe the
// quantum-classical transition.  This version utilizes a large number
// of coins.

#include <math.h>
#include <stdio.h>
#include <iostream.h>
#include <fstream.h>

#include "ACG.h"
#include "Traject.h"
#include "State.h"
#include "Operator.h"
#include "ParticleOp.h"
#include "SpinOp.h"
#include "Complex.h"

#define TWOPI 6.2831853

#include "choose.cc"

int main()
{
// Basic operators

  IdentityOperator id;
  PositionOperator x(0);	// Operates on the zeroth degree of freedom.
  ShiftOperator shift(0);
  SigmaX sx(1);			// Operates on the "coin"
  SigmaY sy(1);
  SigmaZ sz(1);
  SigmaPlus sp(1);

  Operator sm = sp.hc();

// Derived operators

  Operator x2 = x*x;

  Operator proj1 = sp*sm;	// Projector onto |1> for coin
  Operator proj0 = sm*sp;	// Projector onto |0> for coin
  Operator flip = sqrt(0.5)*(sx+sz);	// Hadamard transform for coin flip
  Operator walk = shift*proj0 + shift.hc()*proj1; // Random walk

// Evolution operators

  Operator evolve = walk*flip;
  Operator reset0 = proj0;
  Operator reset1 = sm*proj1;
  const int nPowers = 10;
  Operator evolve0[nPowers];
  Operator evolve1[nPowers];
  evolve0[0] = id;
  evolve1[0] = id;
  int i;
  for(i=1; i<nPowers; i++) {
    evolve0[i] = reset0*evolve.pow(i);
    evolve1[i] = reset1*evolve.pow(i);
  }

// The initial state

  int spaceSize = 70;			// Size of lattice
  State psi1(spaceSize,PARTICLE);	// 1-freedom state ("|x = 0>")
  State psi2(2,SPIN);			// 1-freedom state ("down")
  State psilist[2] = {psi1,psi2};
  State psiIni(2,psilist);		// Product state  ("|0>(|0>^5)")
  //psiIni *= sqrt(0.5)*(id+IM*sx);	// Put the coin in a y eigenstate

// Loop over different numbers of "coins"

  int numSteps = 30;			// Number of iterations of map
  int tDivM, tModM;
  int j,k;

  for(int nCoins=5; nCoins<=5; nCoins++) {

// Loop over the evolution steps

    for( int t=1; t<=numSteps; t++ ) { // iterate over steps

      tDivM = t/nCoins;
      tModM = t%nCoins;
      cout << nCoins << "  " << t << "  " << tDivM << "  " << tModM << endl;
      double xBar=0.0;
      double x2Bar=0.0;
      for( i=0; i<=tModM; i++ )			// Coins flipped tDivM+1 times
      for( j=0; j<=nCoins-tModM; j++) {		// Coins flipped tDivM times
        State psi = psiIni;
        if(i>0) psi *= (reset0*evolve.pow(tDivM+1)).pow(i);
        if(i<tModM) psi *= (reset1*evolve.pow(tDivM+1)).pow(tModM-i);
        if( tDivM > 0 ) {
          if(j>0) psi *= (reset0*evolve.pow(tDivM)).pow(j);
          if(j<nCoins-tModM)
            psi *= (reset1*evolve.pow(tDivM)).pow(nCoins-tModM-j);
        }
        psi *= sqrt(bico(tModM,i)*bico(nCoins-tModM,j));
        xBar += real(psi*(x*psi));
        x2Bar += real(psi*(x2*psi));
      }
      //cout << t << "  " << xBar << "  " << x2Bar-xBar*xBar << endl;
    } // end step loop
    cout << endl;
  }  // end coins loop
}  // end program

